package com.example.fuzzControll.service.impl;

import com.alibaba.fastjson.JSON;
import com.example.fuzzControll.domain.bo.CmdStartParams;
import com.example.fuzzControll.domain.bo.FuzzParams;
import com.example.fuzzControll.domain.bo.TestEntity;
import com.example.fuzzControll.domain.po.Test;
import com.example.fuzzControll.domain.po.TestAndParams;
import com.example.fuzzControll.exception.mysqlException.MysqlException;
import com.example.fuzzControll.mapper.FuzzParamsMapper;
import com.example.fuzzControll.mapper.TestMapper;
import com.example.fuzzControll.service.TestMissionService;
import lombok.Data;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Slf4j
@Service("TestMissionServiceImpl")
public class TestMissionServiceImpl implements TestMissionService {
    @Autowired
    TestMapper testMapper;
    @Autowired
    FuzzParamsMapper fuzzParamsMapper;

    @Override
    public List<TestAndParams> getTestAndParamsList() {
        List<TestAndParams> testParamsList = new ArrayList();
        try {
            testParamsList = testMapper.getTestAndParamsList();
        } catch (Exception e) {
            log.error("Failed to get test and params list:{}", e.getMessage());
            throw new RuntimeException(e);
        }
        if (testParamsList == null || testParamsList.size() == 0) {
            log.warn("TestAndParams is null!");
        }
        return testParamsList;
    }

    @Override
    public boolean deleteById(int id) {
        boolean flag = false;
        try {
            flag = testMapper.deleteById(id);
        } catch (Exception e) {
            log.error("Delete error:{}", e.getMessage());
            throw new RuntimeException(e);
        }
        return false;
    }


    @Override
    public void insert(TestAndParams testAndParams) {
        Test test = new Test(new Date(), new Date(), testAndParams.getTestName());
        testMapper.insert(test);
    }

    @Transactional(rollbackFor = MysqlException.class)
    @Override
    public void insertParmasInTestAflnet(int id, CmdStartParams cmdStartParams) {
        try {
            /*判断test是否存有paramsId,没数据插入有数据就改变*/
            Test test = testMapper.getTestById(id);
            int fuzzParamsId;
            if (test.getFuzzParamsId() == 0) {//为0则插入id
                FuzzParams fuzzParams = new FuzzParams(JSON.toJSONString(cmdStartParams), cmdStartParams.getProtopcol());
                fuzzParamsMapper.insertFuzzParamsReturnId(fuzzParams);
                fuzzParamsId = fuzzParams.getId();
                test.setFuzzParamsId(fuzzParamsId);
            } else {//有则更新数据
                FuzzParams fuzzParams = new FuzzParams(test.getFuzzParamsId(),JSON.toJSONString(cmdStartParams), cmdStartParams.getProtopcol());
                if(!fuzzParamsMapper.editFuzzParams(fuzzParams.getId(),fuzzParams.getParams(),fuzzParams.getProtocol())){
                    throw new RuntimeException("EditFuzzParams failed!");
                }
            }
            /*更新修改时间*/
            test.setUpdateTime(new Date());
            testMapper.edit(test);
        } catch (Exception e) {
            log.error("Failed to insert:[{}]", e.getMessage());
            throw new MysqlException("");
        }
    }

    @Transactional(rollbackFor = MysqlException.class)
    @Override
    public void insertParmasInTestKitty(int id, TestEntity testEntity) {
        /*插入fuzzparams返回id*/
        /*更新对应id的测试数据*/
    }
}
